package com.example.demo.news.service;

import java.util.List;
import java.util.stream.StreamSupport;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.news.model.NewsEntity;
import com.example.demo.news.repository.NewsRepository;

@Service
public class NewsService {
    private final NewsRepository repository;

    public NewsService(NewsRepository repository) {
        this.repository = repository;
    }

    private void checkName(String name) {
        if (repository.findByNameIgnoreCase(name).isPresent()) {
            throw new IllegalArgumentException(
                    String.format("Type with name %s is already exists", name));
        }
    }

    @Transactional(readOnly = true)
    public List<NewsEntity> getAll(Long departmentId) {
        if (departmentId <= 0L) {
            return StreamSupport.stream(repository.findAll().spliterator(), false).toList();
        } else {
            return repository.findByDepartmentId(departmentId);
        }
    }

    @Transactional(readOnly = true)
    public NewsEntity get(Long id) {
        return repository.findById(id)
                .orElseThrow(() -> new NotFoundException(NewsEntity.class, id));
    }

    @Transactional
    public NewsEntity create(NewsEntity entity) {
        checkName(entity.getName());
        return repository.save(entity);
    }

    @Transactional
    public NewsEntity update(Long id, NewsEntity entity) {
        final NewsEntity existsEntity = get(id);
        checkName(entity.getName());
        existsEntity.setDate(entity.getDate());
        existsEntity.setName(entity.getName());
        existsEntity.setDescription(entity.getDescription());
        existsEntity.setDepartment(entity.getDepartment());
        return repository.save(existsEntity);
    }

    @Transactional
    public NewsEntity delete(Long id) {
        final NewsEntity existsEntity = get(id);
        repository.delete(existsEntity);
        return existsEntity;
    }
}
