package com.example.demo.java;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.dao.DataIntegrityViolationException;

import com.example.demo.core.error.NotFoundException;
import com.example.demo.department.model.DepartmentEntity;
import com.example.demo.department.service.DepartmentService;

@SpringBootTest
@TestMethodOrder(OrderAnnotation.class)
class DepartmentsTests {
    @Autowired
    private DepartmentService departmentService;

    private DepartmentEntity department;

    @BeforeEach
    void createData() {
        removeData();

        department = departmentService.create(new DepartmentEntity("ИСТ"));
        departmentService.create(new DepartmentEntity("ГУМ"));
        departmentService.create(new DepartmentEntity("ИЯ"));
    }

    @AfterEach
    void removeData() {
        departmentService.getAll().forEach(item -> departmentService.delete(item.getId()));
    }

    @Test
    void getTest() {
        Assertions.assertThrows(NotFoundException.class, () -> departmentService.get(0L));
    }

    @Test
    void createTest() {
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertEquals(department, departmentService.get(department.getId()));
    }

    @Test
    void createNotUniqueTest() {
        final DepartmentEntity nonUniquedepartment = new DepartmentEntity("гум");
        Assertions.assertThrows(IllegalArgumentException.class, () -> departmentService.create(nonUniquedepartment));
    }

    @Test
    void createNullableTest() {
        final DepartmentEntity nullabledepartment = new DepartmentEntity(null);
        Assertions.assertThrows(DataIntegrityViolationException.class,
                () -> departmentService.create(nullabledepartment));
    }

    @Test
    void updateTest() {
        final String testName = "TESTNAME";
        final String oldName = department.getName();
        final DepartmentEntity newEntity = departmentService.update(department.getId(),
                new DepartmentEntity(testName));
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertEquals(newEntity, departmentService.get(department.getId()));

        Assertions.assertEquals(testName, newEntity.getName());
        Assertions.assertNotEquals(oldName, newEntity.getName());
    }

    @Test
    void deleteTest() {
        departmentService.delete(department.getId());
        Assertions.assertEquals(2, departmentService.getAll().size());

        final DepartmentEntity newEntity = departmentService
                .create(new DepartmentEntity(department.getName()));
        Assertions.assertEquals(3, departmentService.getAll().size());
        Assertions.assertNotEquals(department.getId(), newEntity.getId());
    }
}
